/**
 * This file is part of Gutenberg Tabs.
 *
 * Gutenberg Tabs is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 2 of the License, or any later version.
 *
 * Gutenberg Tabs is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Gutenberg Tabs.  If not, see <https://www.gnu.org/licenses/>.
 */
import classNames from "classnames";
import { __, _x, sprintf } from "@wordpress/i18n";
import { registerBlockType } from '@wordpress/blocks';
import { Button, PanelBody, ResponsiveWrapper, Spinner, TextControl, ToggleControl } from '@wordpress/components';
import {
	InnerBlocks,
	InspectorControls,
	MEDIA_TYPE_IMAGE,
	MediaUpload,
	MediaUploadCheck,
	useBlockProps
} from "@wordpress/block-editor";
import icon from "./icon";
import { useSelect } from "@wordpress/data";
import './editor.css';
import { has } from "lodash";

registerBlockType(
	'tandem/tabs',
	{
		apiVersion: 2,
		title: __( 'Tabs', 'gutenberg-tabs' ),
		icon,
		category: 'design',
		description: __( 'Show only a group of components at a time.', 'gutenberg-tabs' ),
		attributes: {
			activeTab: {
				type: 'number',
				default: 0,
			},
			autoplay: {
				type: 'boolean',
				default: false,
			},
			autoplaySpeed: {
				type: 'int',
				default: 3000,
			},
		},
		providesContext: {
			'tandem/tabs/activeTab': 'activeTab',
		},
		edit( { clientId, attributes, setAttributes } ) {
			const { activeTab, autoplay, autoplaySpeed } = attributes;

			const innerBlocks = useSelect( ( select ) => {
				return select( 'core/block-editor' ).getBlock( clientId ).innerBlocks;
			}, [clientId] );

			const template = [
				[
					'tandem/tab',
					{},
					[],
				],
			];
			return (
				<>
					<InspectorControls>
						<PanelBody
							title={ __( 'Autoplay', 'gutenberg-tabs' ) }
						>
							<ToggleControl
								label={ __( 'Enable autoplay', 'gutenberg-tabs' ) }
								checked={ autoplay }
								onChange={ ( value ) => setAttributes( { autoplay: value } ) }
							/>
							{ autoplay && <TextControl
								label={ __( 'Speed (ms)', 'gutenberg-tabs' ) }
								type="number"
								min="1"
								value={ autoplaySpeed }
								onChange={ ( value ) => setAttributes( { autoplaySpeed: value } ) }
							/> }
						</PanelBody>
					</InspectorControls>
					<div { ...useBlockProps() }>
						<ul className="wp-block-tandem-tabs__selector">
							{ innerBlocks.map( ( tab, index ) => (
								<li className={ classNames( { 'active-tab': activeTab === index } ) }>
									<a
										onClick={ () => {
											setAttributes( {
												activeTab: index,
											} );
										} }
									>{ tab.attributes.label
										? tab.attributes.label
										: sprintf( __( 'Tab %d', 'gutenberg-tabs' ), index + 1 ) }</a>
								</li>
							) ) }
						</ul>
						<div className="wp-block-tandem-tabs__editor">
							<InnerBlocks
								allowedBlocks={ ['tandem/tab'] }
								template={ template }
							/>
						</div>
					</div>
				</>
			);
		},
		save( { attributes } ) {
			const { autoplay, autoplaySpeed } = attributes;
			let attrs = {};
			if ( autoplay ) {
				attrs = {
					'data-autoplay': attributes.autoplaySpeed,
				}
			}
			return (
				<div { ...useBlockProps.save( attrs ) }>
					<div className="wp-block-tandem-tabs__content">
						<InnerBlocks.Content/>
					</div>
				</div>
			);
		},
	}
);

registerBlockType(
	'tandem/tab',
	{
		apiVersion: 2,
		title: __( 'Tab', 'gutenberg-tabs' ),
		icon,
		category: 'design',
		description: __( 'A single tab group.', 'gutenberg-tabs' ),
		attributes: {
			label: {
				type: 'string',
				source: 'attribute',
				selector: '.wp-block-tandem-tab',
				attribute: 'data-label',
			},
			icon: {
				type: 'number',
			},
			iconUrl: {
				type: 'string',
				source: 'attribute',
				selector: '.wp-block-tandem-tab',
				attribute: 'data-icon-url',
			},
			iconWidth: {
				type: 'string',
				source: 'attribute',
				selector: '.wp-block-tandem-tab',
				attribute: 'data-icon-width',
			},
			iconHeight: {
				type: 'string',
				source: 'attribute',
				selector: '.wp-block-tandem-tab',
				attribute: 'data-icon-height',
			},
		},
		parent: [
			'tandem/tabs',
		],
		usesContext: [
			'tandem/tabs/activeTab',
		],
		edit( { clientId, attributes, setAttributes, context } ) {
			const { label, icon } = attributes;

			const instructions =
				<p>{ __( 'To edit the icon, you need permission to upload media.', 'gutenberg-tabs' ) }</p>;

			const { tabIndex, media } = useSelect( ( select ) => {
				const { getMedia } = select( 'core' );
				const editor = select( 'core/block-editor' );
				const rootId = editor.getBlockRootClientId( clientId );
				return {
					tabIndex: editor.getBlockIndex( clientId, rootId ),
					media: icon ? getMedia( icon, { context: 'view' } ) : null,
				};
			}, [clientId, icon] );

			let mediaWidth, mediaHeight, mediaSourceUrl;
			if ( media ) {
				const mediaSize = 'post-thumbnail';
				if ( has( media, ['media_details', 'sizes', mediaSize] ) ) {
					// use mediaSize when available
					mediaWidth = media.media_details.sizes[mediaSize].width;
					mediaHeight = media.media_details.sizes[mediaSize].height;
					mediaSourceUrl = media.media_details.sizes[mediaSize].source_url;
				} else {
					// get fallbackMediaSize if mediaSize is not available
					const fallbackMediaSize = 'thumbnail';
					if ( has( media, ['media_details', 'sizes', fallbackMediaSize] ) ) {
						// use fallbackMediaSize when mediaSize is not available
						mediaWidth = media.media_details.sizes[fallbackMediaSize].width;
						mediaHeight = media.media_details.sizes[fallbackMediaSize].height;
						mediaSourceUrl = media.media_details.sizes[fallbackMediaSize].source_url;
					} else {
						// use full image size when mediaFallbackSize and mediaSize are not available
						mediaWidth = media.media_details.width;
						mediaHeight = media.media_details.height;
						mediaSourceUrl = media.source_url;
					}
				}
				if ( ! media.isLoading ) {
					setAttributes( {
						iconUrl: mediaSourceUrl,
						iconWidth: mediaWidth,
						iconHeight: mediaHeight,
					} );
				}
			}

			const template = [
				[
					'core/paragraph',
					{
						placeholder: _x( 'Content...', 'content placeholder', 'gutenberg-tabs' ),
					},
				],
			];

			const updateIcon = ( image ) => {
				setAttributes( {
					icon: image.id,
				} );
			};

			const removeIcon = () => {
				setAttributes( {
					icon: undefined,
				} );
			};

			const style = {
				display: tabIndex === context['tandem/tabs/activeTab'] ? 'block' : 'none',
			};

			const allowedTypes = [MEDIA_TYPE_IMAGE];

			return (
				<>
					<InspectorControls>
						<PanelBody
							title={ __( 'Tab', 'gutenberg-tabs' ) }
						>
							<TextControl
								label={ __( 'Label', 'gutenberg-tabs' ) }
								value={ label }
								onChange={ ( value ) => setAttributes( { label: value } ) }
							/>
							<div className="editor-post-featured-image">
								<MediaUploadCheck fallback={ instructions }>
									<MediaUpload
										title={ __( 'Select or Upload Icon', 'gutenberg-tabs' ) }
										onSelect={ updateIcon }
										allowedTypes={ allowedTypes }
										modalClass="editor-post-featured-image__media-modal"
										value={ icon }
										render={ ( { open } ) => (
											<div className="editor-post-featured-image__container">
												<Button
													className={ ! icon
														? 'editor-post-featured-image__toggle'
														: 'editor-post-featured-image__preview'
													}
													onClick={ open }
													aria-label={
														! icon
															? null
															: __( 'Edit or update the icon', 'gutenberg-tabs' )
													}
												>
													{ !! icon && media && (
														<ResponsiveWrapper
															naturalWidth={ mediaWidth }
															naturalHeight={ mediaHeight }
															isInline
														>
															<img src={ mediaSourceUrl } alt=""/>
														</ResponsiveWrapper>
													) }
													{ !! icon && ! media && (
														<Spinner/>
													) }
													{ ! icon && __( 'Set icon', 'gutenberg-tabs' ) }
												</Button>
											</div>
										) }
									/>
								</MediaUploadCheck>
								{ !! icon && media && ! media.isLoading && (
									<MediaUploadCheck>
										<MediaUpload
											title={ __( 'Replace Icon', 'gutenberg-tabs' ) }
											onSelect={ updateIcon }
											allowedTypes={ allowedTypes }
											modalClass="editor-post-featured-image__media-modal"
											render={ ( { open } ) => (
												<Button onClick={ open } isSecondary>
													{ __( 'Replace icon', 'gutenberg-tabs' ) }
												</Button>
											) }
										/>
									</MediaUploadCheck>
								) }
								{ !! icon && (
									<MediaUploadCheck>
										<Button
											onClick={ removeIcon }
											isLink
											isDestructive
										>
											{ __( 'Remove icon', 'gutenberg-tabs' ) }
										</Button>
									</MediaUploadCheck>
								) }
							</div>
						</PanelBody>
					</InspectorControls>
					<div { ...useBlockProps( { style } ) }>
						<InnerBlocks
							template={ template }
						/>
					</div>
				</>
			);
		},
		save( { attributes } ) {
			const attrs = {
				'data-icon-url': attributes.iconUrl,
				'data-icon-width': attributes.iconWidth,
				'data-icon-height': attributes.iconHeight,
				'data-label': attributes.label,
			}
			return (
				<div { ...useBlockProps.save( attrs ) } >
					<InnerBlocks.Content/>
				</div>
			);
		},
	}
);
